#include "Display.h"
#include <iostream>


// dclaration du pointeur statique de l'instance unique du gestionnaire de fentrage
CDisplay* CDisplay::m_pInstance = NULL;


// constructeur & destructeur virtuel
CDisplay::CDisplay(SParameters* pParams)
{
    // vrification de l'initialisation video de la SDL
    if (!SDL_WasInit( SDL_INIT_VIDEO ))
		throw "SDL_VIDEO non initialise"; // exception

	// mise  jour de l'affichage avec cration initiale de la fentre
	// si des paramtres ont t transmis on active l'affichage avec ceux l
	// si pas de paramtres transmis ou chec avec les paramtres transmis
	// on utilise les paramtres par dfaut
	if(!pParams || !Update(*pParams))
	{
		CDisplay::SParameters params = DefaultParameters();
		if(!Update(params))
			throw "Echec lors de l'initialisation de l'affichage";
	}
	// paramtres temporaires = paramtres courants
	ResetTemp();
}

CDisplay::~CDisplay(void)
{
	// rien  faire dans le destructeur
}


// mthode statique d'initialisation du singleton
CDisplay* CDisplay::Initialize(SParameters* pParams /*= NULL*/)
{
	if(!m_pInstance)
	{
		m_pInstance = new CDisplay(pParams);

		if(!m_pInstance)
			throw "La creation de CDisplay a echouee";
	}

	return m_pInstance;
}

// mthode statique de libration du singleton
void CDisplay::ShutDown(void)
{
	if(m_pInstance)
		delete m_pInstance;

	m_pInstance = NULL;
}


// modification des paramtres d'affichage
void CDisplay::ScreenSize(const int nScreenX, const int nScreenY, const int nScreenBpp /*= 0*/)
{
	m_sTemp.nScreenX = nScreenX;
	m_sTemp.nScreenY = nScreenY;
	if(nScreenBpp > 0)
		ScreenBpp(nScreenBpp);
}


// basculement du mode fentr en mode plein cran et vice-versa
const bool CDisplay::SwitchFullscreen(void)
{
	// rcupration des paramtres d'affichage actuels
	ResetTemp();

	// modification de l'indicateur plein cran
	m_sTemp.bFullscreen = !m_sCurrent.bFullscreen;

	// mise  jour de l'affichage
	return Update();
}


// retourne les paramtres par dfaut
CDisplay::SParameters CDisplay::DefaultParameters(void)
{
	SParameters sParams;

	// paramtres d'affichage
	sParams.nScreenX = DISPLAY_SCREENX_DEFAULT;
	sParams.nScreenY = DISPLAY_SCREENY_DEFAULT;
	sParams.nScreenBpp = DISPLAY_SCREENBPP_DEFAULT;
	// indicateur d'affichage plein cran
	sParams.bFullscreen = DISPLAY_FULLSCREEN_DEFAULT;
	// paramtres OpenGL
	sParams.nZBits = DISPLAY_ZBUFFER_DEFAULT;
	sParams.nStencilBits = DISPLAY_STENCIL_DEFAULT;

	return sParams;
}

// mise  jour de l'affichage
bool CDisplay::Update(void)
{
	// on essaye de mettre  jour l'affichage avec les paramtres temporaires
	if(Update(m_sTemp))
		return true; // on retourne true pour signaler que la modification a russie

	// on a chou une fois, on va tenter de revenir aux anciens paramtres
	if(Update(m_sCurrent))
		return false; // c'est russi, on retourne false pour prvenir que les nouveaux paramtres sont invalides

	// on n'a pas russi  revenir aux paramtres prcdents, on lance une exception
	throw "Les nouveaux parametres sont invalides et le retour aux precedents a echoue";

	// on ne devrait jamais arriver ici
	return false;
}

// mise  jour de l'affichage avec les paramtres choisis
bool CDisplay::Update(SParameters& params)
{
	// l'indicateur SDL doit tre paramtr sur SDL_OPENGL pour activer une surface OpenGL
	Uint32 flags = SDL_OPENGL ;
	if(params.bFullscreen) // si l'indicateur plein cran est actif
		flags |= SDL_FULLSCREEN; // on ajoute l'indicateur SDL_FULLSCREEN

	// modification de la surface d'affichage SDL
	if(!SDL_SetVideoMode(params.nScreenX, params.nScreenY, params.nScreenBpp, flags))
        return false;

    // activation du double buffer pour OpenGL
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	// paramtrage du ZBuffer
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, params.nZBits);

	// paramtrage du StencilBuffer
	SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, params.nStencilBits);

	// russi, on mmorise les paramtres
	if(&params != &m_sCurrent)
		m_sCurrent = params;

	// et on retourne true
	return true;
}
