#ifndef DISPLAY_CLASS_HEADER
#define DISPLAY_CLASS_HEADER


// includes
#include <SDL/SDL.h>

// defines
#define DISPLAYMANAGER					(*CDisplay::Instance())


// constantes
#ifdef DEBUG
const int DISPLAY_SCREENX_DEFAULT 	=		640;
const int DISPLAY_SCREENY_DEFAULT	=		480;
const int DISPLAY_SCREENBPP_DEFAULT	=		16;
const bool DISPLAY_FULLSCREEN_DEFAULT	=	false;
const int DISPLAY_ZBUFFER_DEFAULT	=		16;
const int DISPLAY_STENCIL_DEFAULT	=		0;
#else
const int DISPLAY_SCREENX_DEFAULT 	=		800;
const int DISPLAY_SCREENY_DEFAULT	=		600;
const int DISPLAY_SCREENBPP_DEFAULT	=		32;
const bool DISPLAY_FULLSCREEN_DEFAULT	=		true;
const int DISPLAY_ZBUFFER_DEFAULT	=		24;
const int DISPLAY_STENCIL_DEFAULT	=		0;
#endif // __DEBUG


// singleton de gestion de l'affichage avec SDL
class CDisplay
{
public:
	// structure pour les paramtres d'affichage
	struct SParameters {
		// paramtres d'affichage
		int		nScreenX;
		int		nScreenY;
		int		nScreenBpp;
		// indicateur plein cran
		bool	bFullscreen;
		// paramtres OpenGL
		int		nZBits;
		int		nStencilBits;
	};


private:
	// constructeur & destructeur virtuel
	CDisplay(SParameters* pParams);
	virtual ~CDisplay(void);

	// prototypes singleton
	CDisplay(const CDisplay&);
	const CDisplay& operator=(const CDisplay&);

public:
	// mthodes statiques pour crer et rcuprer l'instance unique du gestionnaire
	static CDisplay* Initialize(SParameters* pParams = NULL);
	static CDisplay* Instance(void) { return m_pInstance; }
	static void ShutDown(void);

	// modification et rcupration des paramtres d'affichage
	void ScreenSize(const int nScreenX, const int nScreenY, const int nScreenBpp = 0);
	void ScreenBpp(const int nScreenBpp) { m_sTemp.nScreenBpp = nScreenBpp; }
	const int ScreenX(void) const { return m_sCurrent.nScreenX; }
	const int ScreenY(void) const { return m_sCurrent.nScreenY; }
	const int ScreenBpp(void) const { return m_sCurrent.nScreenBpp; }

	// modification et rcupration du paramtre plein cran
	void Fullscreen(const bool bFullscreen) { m_sTemp.bFullscreen = bFullscreen; }
	const bool Fullscreen(void) const { return m_sCurrent.bFullscreen; }
	const bool SwitchFullscreen(void);

	// modification et rcupration des paramtres OpenGL
	void ZBuffer(const int nZbits) { m_sTemp.nZBits = nZbits; }
	const int ZBuffer(void) const { return m_sCurrent.nZBits; }
	void Stencil(const int nStencilbits) { m_sTemp.nStencilBits = nStencilbits; }
	const int Stencil(void) const { return m_sCurrent.nStencilBits; }

	// rcuprateurs des structures contenant tous les paramtres
	const SParameters& GetCurrent(void) const { return m_sCurrent; }
	SParameters& GetTemp(void) { return m_sTemp; }

	// remet les paramtres temporaires aux valeurs des paramtres courants
	void ResetTemp(void) { m_sTemp = m_sCurrent; }

	// retourne les paramtres par dfaut
	SParameters DefaultParameters(void);

	// mise  jour de l'affichage
	bool Update(void);


protected:
	// mise  jour de l'affichage avec les paramtres choisis
	bool Update(SParameters& params);


protected:
	// pointeur pour l'instance unique du gestionnaire
	static CDisplay*		m_pInstance;
	// paramtres concernant l'affichage actuel
	SParameters				m_sCurrent;
	// paramtres temporaires utiliss lors d'un Update()
	SParameters				m_sTemp;
};


#endif // DISPLAY_CLASS_HEADER
