#include "Main.h"
#include "TimerSps.h"

void ResizeViewport(void)
{
	// affichage dans la console de la rsolution actuelle
	std::cout << "\nResolution " << (DISPLAYMANAGER.Fullscreen()?"Plein ecran : ":"Fenetre : ") << DISPLAYMANAGER.ScreenX() << "x" << DISPLAYMANAGER.ScreenY() << "x" << DISPLAYMANAGER.ScreenBpp() << std::endl;

	glViewport(0, 0, DISPLAYMANAGER.ScreenX(), DISPLAYMANAGER.ScreenY());
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, (GLfloat)DISPLAYMANAGER.ScreenX()/(GLfloat)DISPLAYMANAGER.ScreenY(), 0.1f, 100.0f);
	glMatrixMode(GL_MODELVIEW);
}

void ResizeWindow(int largeur, int hauteur)
{
	DISPLAYMANAGER.ScreenSize(largeur, hauteur);
	if(!DISPLAYMANAGER.Update())
		std::cerr << "Echec lors du passage en " << largeur << "x" << hauteur << std::endl;
	ResizeViewport();
}

int main ( int argc, char** argv )
{
	try {
		if(SDL_Init(SDL_INIT_VIDEO) < 0)
			throw "SDL_Init(SDL_INIT_VIDEO) a echoue !!!";

		// initialisation du fentrage
        std::cerr << "In main" << std::endl;
		CDisplay::Initialize();
        std::cerr << "In main" << std::endl;
		ResizeViewport();

		float zRot = 0; // une petite variable pour l'animation
		CTimerSps ticks; // le calculateur de temps pour le nombre d'images par seconde
		bool bRunning = true;
		// boucle principale du programme
		while (bRunning)
		{
			SDL_Event event; // un vnement SDL
			// rcupration d'un vnement dans la pile
			while (SDL_PollEvent(&event))
			{
				// analyse du message
				switch(event.type)
				{
					// message demandant  quitter
					case SDL_QUIT:
						bRunning = false;
						break;
						// message signalant l'appui sur une touche
					case SDL_KEYDOWN:
						switch(event.key.keysym.sym) // symbole de la touche appuye
						{
							case SDLK_ESCAPE: // touche ECHAP pour quitter
								bRunning = false;
								break;
							case SDLK_SPACE: // touche ESPACE pour passer en plein cran
								DISPLAYMANAGER.SwitchFullscreen();
								ResizeViewport();
								break;
							case SDLK_F1: // changements de rsolution
								ResizeWindow(640,480);
								break;
							case SDLK_F2:
								ResizeWindow(800,600);
								break;
							case SDLK_F3:
								DISPLAYMANAGER.ScreenBpp(32);
								ResizeWindow(1024,768);
								break;
							case SDLK_F4:
								DISPLAYMANAGER.ScreenBpp(32);
								ResizeWindow(1280,1024);
								break;
							case SDLK_F5: // rsolution par dfaut
								DISPLAYMANAGER.GetTemp() = DISPLAYMANAGER.DefaultParameters();
								if(!DISPLAYMANAGER.Update())
									std::cout << "Echec lors du passage en paramtres par dfaut" << std::endl;
								ResizeViewport();
								break;
                            default: 
                                break;
						} // fin du switch(event.key.keysym.sym)
						break;
                     default:
                        break;
				} // fin du switch(event.type)
			} // fin de la boucle de gestion des vnements

			// effacement de l'cran et du ZBuffer
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

			// on affiche un beau triangle histoire d'avoir quelque chose  voir
			glLoadIdentity();
			glTranslatef(0.0f, 0.0f, -4.0f);
			glRotatef(zRot, 0, 0, 1);
			glBegin(GL_TRIANGLES);
			glColor3f(1.0f,0.0f,0.0f);
			glVertex3f( 0.0f, 1.0f, 0.0f);
			glColor3f(0.0f,1.0f,0.0f);
			glVertex3f(-1.0f,-1.0f, 0.0f);
			glColor3f(0.0f,0.0f,1.0f);
			glVertex3f( 1.0f,-1.0f, 0.0f);
			glEnd();

			// change des buffers back et front
			SDL_GL_SwapBuffers();

			// un peu d'animation, le triangle va faire un tour par seconde
			zRot += ticks.Step() / (1000.f / 360.f);

			// on calcule le nombre d'images par secondes 2 fois par seconde
			if(ticks.TotalTime() > 100)
			{
				std::cout << "\r" << std::setw(3) << ticks.Compute() << " images par seconde" << std::ends;
				ticks.Restart();
			}
		} // fin de la boucle principale du programme
	}
	catch(char* msg)
	{
		std::cerr << msg << std::endl;
	}

	if(CDisplay::Instance())
		CDisplay::ShutDown();

	SDL_Quit();

	// hitoire d'viter les warning parceque argc et argv ne sont pas utiliss
	(void) argc;
	(void) argv;

	return 0;
}

